@@ -7,11 +7,11 @@ Posy::Plugin::MultiEntry - Posy plugin to enable multiple entries to display for
 
 =head1 VERSION
 
-This describes version B<0.02> of Posy::Plugin::MultiEntry.
+This describes version B<0.03> of Posy::Plugin::MultiEntry.
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -27,18 +27,16 @@ like special "category" pages; Posy will display not only that entry,
 but all the entries which would normally be selected when that category
 is displayed.  This can be set up using a per-file config file,
 which would enable one to use different settings for just that page,
-such as sorting in a different order, and so on.
+such as sorting in a different order, showing a special sub-set
+of files, and so on.
 
-This plugin replaces the 'select_entries' and 'sort_entries' actions.  When
-using this plugin in conjunction with others which override 'sort_entries'
-(such as L<Posy::Plugin::Info>), this plugin should go later in the plugins
-list than them, otherwise its sort will be overridden.
+This plugin replaces the 'select_entries' action.
 
 =head2 Configuration
 
 This expects configuration settings in the $self->{config} hash,
 which, in the default Posy setup, can be defined in the main "config"
-file in the data directory.
+file in the config directory.
 
 =over
 
@@ -47,18 +45,17 @@ file in the data directory.
 Turn on multi-entry selection if the path-type is an entry.
 (default: false)
 
-=item B<multi_entry_change_type>
+=item B<multi_entry_exclude>
 
-If multi_entry_on is true and multi_entry_change_type is true,
-change the path-type to be 'category' or 'top_category'.
-(default: true)
+Filter out the entries in the current category which match this pattern.
+Note that one can actually set this to exclude the entry that
+the multi_entry selection is being done with!
 
-=item B<multi_entry_place_first>
+=item B<multi_entry_include>
 
-If true, (and multi_entry_on) this will place the original entry first in
-the displayed list of entries.  This could be used to make that entry
-contain introductory information, followed by all the other entries.
-(default: false)
+Only keep the entries in the current category which match this pattern.
+An entry has to both match the include pattern and not match the exclude
+pattern for it to be included.
 
 =back
 
@@ -80,10 +77,10 @@ sub init {
     # set defaults
     $self->{config}->{multi_entry_on} = 0
 	if (!defined $self->{config}->{multi_entry_on});
-    $self->{config}->{multi_entry_change_type} = 1
-	if (!defined $self->{config}->{multi_entry_change_type});
-    $self->{config}->{multi_entry_place_first} = 0
-	if (!defined $self->{config}->{multi_entry_place_first});
+    $self->{config}->{multi_entry_exclude} = ''
+	if (!defined $self->{config}->{multi_entry_exclude});
+    $self->{config}->{multi_entry_include} = '.*'
+	if (!defined $self->{config}->{multi_entry_include});
 } # init
 
 =head1 Flow Action Methods
@@ -99,8 +96,12 @@ $self->select_entries($flow_state);
 
 Select entries by looking at the path information.
 Assumes that no entries have been selected before.
-Sets $flow_state->{entries}.  Assumes it hasn't
-already been set.
+
+If this is an entry, and multi_entry_on is true, then
+selects all entries as if this were a category, and sets
+the path type to be category.
+
+Otherwise, calls the parent method.
 
 =cut
 sub select_entries {
@@ -111,30 +112,42 @@ sub select_entries {
 	or $self->{path}->{type} eq 'top_entry')
 	and $self->{config}->{multi_entry_on})
     {
-	# select as for category
-	if ($self->{path}->{cat_id} eq '') 
+	# we are doing multi-entry!
+	$self->debug(2, "MultiEntry: " . $self->{path}->{file_key});
+	# select entries as for category
+	# skipping the excluded entries
+	# including the included entries
+	$flow_state->{entries} = [];
+	foreach my $key (keys %{$self->{files}})
 	{
-	    @{$flow_state->{entries}} = keys %{$self->{files}};
-	    $self->{path}->{type} = 'top_category'
-		if $self->{config}->{multi_entry_change_type};
-	}
-	else
-	{
-	    $flow_state->{entries} = [];
-	    foreach my $key (keys %{$self->{files}})
+	    if (!($self->{config}->{multi_entry_exclude}
+		  and $key =~ m#$self->{config}->{multi_entry_exclude}#
+		 )
+		  and $key =~ m#$self->{config}->{multi_entry_include}#
+	       )
 	    {
-		if (($self->{files}->{$key}->{cat_id} 
-		     eq $self->{path}->{cat_id}) # the same directory
-		    or ($self->{files}->{$key}->{cat_id} 
-			=~ m#^$self->{path}->{cat_id}/#) # a subdir
+		if (($self->{path}->{cat_id} eq '')
+		    or (($self->{files}->{$key}->{cat_id} 
+			 eq $self->{path}->{cat_id}) # the same directory
+			or ($self->{files}->{$key}->{cat_id} 
+			    =~ m#^$self->{path}->{cat_id}/#)) # a subdir
 		   )
 		{
 		    push @{$flow_state->{entries}}, $key;
 		}
 	    }
-	    $self->{path}->{type} = 'category'
-		if $self->{config}->{multi_entry_change_type};
 	}
+
+	# reset the path type to 'category' and reset the config
+	if ($self->{path}->{cat_id} eq '') 
+	{
+	    $self->{path}->{type} = 'top_category';
+	}
+	else
+	{
+	    $self->{path}->{type} = 'category';
+	}
+	$self->set_config($flow_state);
     }
     else
     {
@@ -142,53 +155,6 @@ sub select_entries {
     }
 } # select_entries
 
-=head2 sort_entries
-
-$self->sort_entries($flow_state);
-
-If the config variable multi_entry_place_first is true,
-extract the original entry from the list, sort the other
-entries with the parent 'sort_entries' method, and
-place the original entry first in the list.
-
-Otherwise just use the parent sort.
-
-=cut
-sub sort_entries {
-    my $self = shift;
-    my $flow_state = shift;
-
-    # no point sorting if there's only one
-    if (@{$flow_state->{entries}} > 1)
-    {
-	if ($self->{config}->{multi_entry_on}
-	    and $self->{config}->{multi_entry_place_first}
-	    and exists $self->{files}->{$self->{path}->{file_key}})
-	{
-	    # remove the original entry-id from the list
-	    my @entries = @{$flow_state->{entries}};
-	    $flow_state->{entries} = [];
-	    while (@entries)
-	    {
-		my $id = shift @entries;
-		if ($id ne $self->{path}->{file_key})
-		{
-		    push @{$flow_state->{entries}}, $id;
-		}
-	    }
-	    # sort the list
-	    $self->SUPER::sort_entries($flow_state);
-	    # put the original entry at the front of the list
-	    unshift @{$flow_state->{entries}}, $self->{path}->{file_key};
-	}
-	else
-	{
-	    $self->SUPER::sort_entries($flow_state);
-	}
-    }
-    1;	
-} # sort_entries
-
 =head1 INSTALLATION
 
 Installation needs will vary depending on the particular setup a person
